#include "qmc.h"
#include "AsianOption.h"

#define NUMBLOCKS 2
#define INTLOOP 0
#define EXTLOOP 1

// ----------------------------------------------------------------------
int main (int argc, char **argv)
{  
  int i, j, m, n, s;
  int k;
		
  double *point = NULL;
  double *rvector = NULL;
  double *rpoint = NULL;
  double result = 0.0;

  STATSQMC stats = NULL;
  CONFIDENCEINTERVAL cf;

  /*double K = 100;
  double T1 = 0.246576;
  double S0 = 100.0;
  double r = 0.086177;
  double T = 0.328767;
  double sigma = 0.2;*/

  double K = 110;
  double T1 = 0;
  double S0 = 100.0;
  double r = 0.05;
  double T = 1;
  double sigma = 0.3;

  if(!LoadInputQMC(argv[1]))
  {
     printf ("Cannot load the input file.\n");
     return (0);
  }

  if (!InitQMC ())
  {
     printf ("Cannot initialize the Quasi Monte Carlo method.\n");
     return (0);
  }

  m = GetQMC(QMC_RANDS);
  n = GetQMC(QMC_NPOINTS);
  s = GetQMC(QMC_DIMEN);

  if ((stats = InitStat (NUMBLOCKS)) == NULL)
  {
     printf ("Cannot initialize stats.\n");
     FreeQMC();
     return (0);
  }
	
  for (i = 0; i < m; i++)
  {
     ResetStat (stats, INTLOOP);
     ResetQMC();

     rvector = GenRandom();

     for (j = 0; j < n; j++)
     {
       if ((point = QMC()) == NULL)
       {
	  printf ("ERROR cannot generate point %d.\n", j);
	  FreeQMC();
	  FreeStat (stats);
	  return (0);
       }
       
       

       if ((rpoint = AppRandom (point, rvector)) == NULL)
       {
	  printf ("Cannot apply the randomization.\n");
	  FreeQMC();
	  FreeStat (stats);
	  return (0);
       }

       result = Option (K, S0, r, T1, T, sigma, s, rpoint);
       StatUpdate1 (stats, INTLOOP, result);
     }
      
     result = StatAverage (stats, INTLOOP, RANDVARONE);
     StatUpdate1 (stats, EXTLOOP, result);
  }
 
  printf ("Average: %f\n", StatAverage (stats, EXTLOOP, RANDVARONE));
  printf ("Variance: %f\n", StatVariance (stats, EXTLOOP, RANDVARONE));
  cf = StatConfIntval (stats, EXTLOOP, RANDVARONE, 0.95);
  printf ("Confidence Interval (95%%): %f to %f\n", cf.low, cf.high);

  FreeQMC();
  FreeStat (stats);
  return (1);
}

// ----------------------------------------------------------------------
